// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2022 Florian Schmaus
#pragma once

#include <ostream>

#include "emper-common.h"

class Fiber;
class Fibril;

class AbstractFiber {
	friend class Dispatcher;

 protected:
	virtual ~AbstractFiber() = default;

	virtual void run() = 0;

 public:
	virtual auto isRunnable() const -> bool { return true; }

	virtual auto getAffinityBuffer() const -> workeraffinity_t* { return nullptr; }

	virtual auto asFiberIfPossible() -> Fiber* { return nullptr; };
	virtual auto asFibrilIfPossible() -> Fibril* { return nullptr; };

	virtual void printTo(std::ostream& strm, bool withPtr = true) const = 0;

	friend auto operator<<(std::ostream& strm, const AbstractFiber& fiber) -> std::ostream&;
	friend auto operator<<=(std::ostream& strm, const AbstractFiber& fiber) -> std::ostream&;
};
