// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021-2022 Florian Fischer, Florian Schmaus
#pragma once

#include <optional>
#include <vector>

#include "Runtime.hpp"
#include "emper-common.h"

class RuntimeBuilder {
 private:
	workerid_t workerCount = 0;

	std::vector<Runtime::NewWorkerHook> newWorkerHooks;

	std::optional<Runtime::StrategyFactory> strategyFactory = std::nullopt;

	std::optional<bool> pinWorkers = std::nullopt;
	std::optional<workerid_t> pinningOffset = std::nullopt;

	std::optional<unsigned> seed = std::nullopt;

 public:
	inline auto withWorkerCount(workerid_t workerCount) -> RuntimeBuilder& {
		this->workerCount = workerCount;
		return *this;
	};

	inline auto newWorkerHook(const Runtime::NewWorkerHook& hook) -> RuntimeBuilder& {
		newWorkerHooks.push_back(hook);
		return *this;
	};

	inline auto withPinWorkers(bool pinWorkers) -> RuntimeBuilder& {
		this->pinWorkers = pinWorkers;
		return *this;
	};

	inline auto withPinningOffset(workerid_t pinningOffset) -> RuntimeBuilder& {
		this->pinningOffset = pinningOffset;
		return *this;
	};

	inline auto withStrategyFactory(Runtime::StrategyFactory strategyFactory) -> RuntimeBuilder& {
		this->strategyFactory = strategyFactory;
		return *this;
	};

	inline auto withSeed(unsigned seed) -> RuntimeBuilder& {
		this->seed = seed;
		return *this;
	};

	inline auto build() -> Runtime {
		auto workerCount =
				this->workerCount != 0 ? this->workerCount : Runtime::getDefaultWorkerCount();

		auto strategyFactory = this->strategyFactory.value_or(Runtime::DEFAULT_STRATEGY_FACTORY);

		auto pinWorkers = this->pinWorkers.value_or(Runtime::shouldPinWorkers());

		auto pinningOffset = this->pinningOffset.value_or(Runtime::getDefaultPinningOffset());

		auto seed = this->seed.value_or(std::random_device()());

		return Runtime(workerCount, newWorkerHooks, pinWorkers, pinningOffset, strategyFactory, seed);
	}
};
