// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021-2022 Florian Fischer
#pragma once

#include <atomic>
#include <cstddef>
#include <string>

namespace emper::lib {
class MappedFileBuffer {
	// We use a triple buffer scheme, where one buffer is active, one is old and one is fresh
	// The old buffer gets remapped by the first accessing the fresh buffer
	static const int BUFFER_COUNT = 3;
	// Use 1GB buffers to reduce the risk of using an unmapped one
	static const size_t BUFFER_SIZE = 1 << 30;

	char* bufs[BUFFER_COUNT];

	std::atomic<size_t> bufPos = 0;

	std::string file;
	int fd;

	auto getBuf(size_t pos) -> char* { return bufs[(pos / BUFFER_SIZE) % BUFFER_COUNT]; }

	void trim();

 public:
	MappedFileBuffer(const std::string& file);
	~MappedFileBuffer();

	void put(const std::string& message);
};
}	 // namespace emper::lib
