// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#pragma once

#include <cstddef>
#include <iostream>
#include <memory>
#include <mutex>

#include "Emper.hpp"
#include "sleep_strategy/WorkerStats.hpp"

class Runtime;

template <typename D>
class WorkerLocalData;

// macro to safely increment the worker local or the anywhere stats
#define __EMPER_SS_STATS_ADD(__field__, count)              \
	if constexpr (emper::STATS) {                             \
		if (stats) {                                            \
			stats->__field__ += count;                            \
		} else {                                                \
			std::lock_guard<std::mutex> lock(anywhereStatsMutex); \
			anywhereStats.__field__ += count;                     \
		}                                                       \
	}

namespace emper::sleep_strategy {

// Somehow iwyu insists on this rather silly looking forward declaration.
// I assume because of the friend declaration in WorkerStats.hpp.
// But using a forward declaration there did not please iwyu.
class Stats;

class Stats {
	WorkerStats anywhereStats;
	std::mutex anywhereStatsMutex;

	static thread_local WorkerStats* stats;
	std::unique_ptr<WorkerLocalData<WorkerStats>> allStats;

 public:
	Stats(Runtime& runtime);
	~Stats();

	void print(std::ostream& out);

	void addOnNewWork(size_t count) { __EMPER_SS_STATS_ADD(onNewWork, count); }
	void incOnNewWork() { addOnNewWork(1); }
	void addNotify(size_t count) { __EMPER_SS_STATS_ADD(notify, count); }
	void incNotify() { addNotify(1); }
	void addNotifications(size_t count) { __EMPER_SS_STATS_ADD(notifications, count); }
	void incNotifications() { addNotifications(1); }
	void addSkip(size_t count) { __EMPER_SS_STATS_ADD(skip, count); }
	void incSkip() { addSkip(1); }
	void addWakeup(size_t count) { __EMPER_SS_STATS_ADD(wakeup, count); }
	void incWakeup() { addWakeup(1); }
	void addWakeupDueToNotify(size_t count) { __EMPER_SS_STATS_ADD(wakeupDueToNotify, count); }
	void incWakeupDueToNotify() { addWakeupDueToNotify(1); }
};
}	 // namespace emper::sleep_strategy
