// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2022 Florian Schmaus
#pragma once

#include <functional>
#include <memory>

#include "WorkerLocalData.hpp"
#include "strategies/AbstractWorkStealingStrategy.hpp"
#include "strategies/laws/LawsDispatcher.hpp"
#include "strategies/laws/LawsScheduler.hpp"

class LawsStrategyFactory;
class LawsStrategyStats;
class Runtime;
class RuntimeStrategyStats;
struct LawsWorkerStats;

class LawsStrategy : public AbstractWorkStealingStrategy {
 private:
	LawsScheduler scheduler;
	LawsDispatcher dispatcher;

	static thread_local LawsWorkerStats stats;

	WorkerLocalData<LawsWorkerStats> allWorkerStats;

	LawsStrategy(Runtime& runtime);

	auto getScheduler() -> LawsScheduler& override;

	auto getDispatcher() -> LawsDispatcher& override;

 public:
	static auto getFactory() -> std::function<LawsStrategy*(Runtime&)>;

	auto getStats() -> std::shared_ptr<RuntimeStrategyStats> override;

	friend LawsScheduler;
	friend LawsDispatcher;
	friend LawsStrategyStats;
	friend LawsStrategyFactory;
};
